@@ -1,6 +1,30 @@
 from .resource import Resource
 from .account import Account
 
+#
+# The MIT License (MIT)
+# 
+# Copyright (C) 2014 hellosign.com
+# 
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+#
+
 class Team(Resource):
     ''' Contains information about your team and its members
 
@@ -14,8 +38,10 @@ class Team(Resource):
 
             account_id (str): The id of the Account
             email_address (str): The email address associated with the Account
-            role_code (str): The membership role for the team. O = Owner,
-                M = Member
+            role_code (str): The membership role for the team. 
+                a = Admin,
+                m = Member
+                d = Developer
 
         invited_accounts (str): A list of all Accounts that have an outstanding
             invitation to join your Team. Note that this response is a subset
@@ -26,7 +52,7 @@ class Team(Resource):
 
     '''
 
-    def __init__(self, jsonstr=None, key=None):
+    def __init__(self, jsonstr=None, key=None, warnings=None):
         ''' Initialization of the object
 
         Args:
@@ -36,8 +62,9 @@ class Team(Resource):
                 of the object
             key (str): Optional key to use with jsonstr. If `key` exists, we'll
                 load the data of `jsonstr[key]` instead of the whole `jsonstr`
+            warnings (list): List of associated warnings
         '''
-        super(Team, self).__init__(jsonstr, key)
+        super(Team, self).__init__(jsonstr, key, warnings)
         if 'accounts' in self.json_data:
             acct_list = []
             for acct in self.accounts:
